@@ -3,18 +3,18 @@
 
 # ***********************IMPORTANT NMAP LICENSE TERMS************************
 # *                                                                         *
-# * The Nmap Security Scanner is (C) 1996-2015 Insecure.Com LLC. Nmap is    *
-# * also a registered trademark of Insecure.Com LLC.  This program is free  *
-# * software; you may redistribute and/or modify it under the terms of the  *
-# * GNU General Public License as published by the Free Software            *
-# * Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE CLARIFICATIONS  *
-# * AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your right to use,    *
-# * modify, and redistribute this software under certain conditions.  If    *
-# * you wish to embed Nmap technology into proprietary software, we sell    *
-# * alternative licenses (contact sales@nmap.com).  Dozens of software      *
-# * vendors already license Nmap technology such as host discovery, port    *
-# * scanning, OS detection, version detection, and the Nmap Scripting       *
-# * Engine.                                                                 *
+# * The Nmap Security Scanner is (C) 1996-2016 Insecure.Com LLC ("The Nmap  *
+# * Project"). Nmap is also a registered trademark of the Nmap Project.     *
+# * This program is free software; you may redistribute and/or modify it    *
+# * under the terms of the GNU General Public License as published by the   *
+# * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
+# * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
+# * right to use, modify, and redistribute this software under certain      *
+# * conditions.  If you wish to embed Nmap technology into proprietary      *
+# * software, we sell alternative licenses (contact sales@nmap.com).        *
+# * Dozens of software vendors already license Nmap technology such as      *
+# * host discovery, port scanning, OS detection, version detection, and     *
+# * the Nmap Scripting Engine.                                              *
 # *                                                                         *
 # * Note that the GPL places important restrictions on "derivative works",  *
 # * yet it does not provide a detailed definition of that term.  To avoid   *
@@ -56,12 +56,19 @@
 # * particularly including the GPL Section 3 requirements of providing      *
 # * source code and allowing free redistribution of the work as a whole.    *
 # *                                                                         *
-# * As another special exception to the GPL terms, Insecure.Com LLC grants  *
+# * As another special exception to the GPL terms, the Nmap Project grants  *
 # * permission to link the code of this program with any version of the     *
 # * OpenSSL library which is distributed under a license identical to that  *
 # * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
 # * linked combinations including the two.                                  *
 # *                                                                         *
+# * The Nmap Project has permission to redistribute Npcap, a packet         *
+# * capturing driver and library for the Microsoft Windows platform.        *
+# * Npcap is a separate work with it's own license rather than this Nmap    *
+# * license.  Since the Npcap license does not permit redistribution        *
+# * without special permission, our Nmap Windows binary packages which      *
+# * contain Npcap may not be redistributed without special permission.      *
+# *                                                                         *
 # * Any redistribution of Covered Software, including any derived works,    *
 # * must obey and carry forward all of the terms of this license, including *
 # * obeying all GPL rules and restrictions.  For example, source code of    *
@@ -101,12 +108,12 @@
 # * to the dev@nmap.org mailing list for possible incorporation into the    *
 # * main distribution.  By sending these changes to Fyodor or one of the    *
 # * Insecure.Org development mailing lists, or checking them into the Nmap  *
-# * source code repository, it is understood (unless you specify otherwise) *
-# * that you are offering the Nmap Project (Insecure.Com LLC) the           *
-# * unlimited, non-exclusive right to reuse, modify, and relicense the      *
-# * code.  Nmap will always be available Open Source, but this is important *
-# * because the inability to relicense code has caused devastating problems *
-# * for other Free Software projects (such as KDE and NASM).  We also       *
+# * source code repository, it is understood (unless you specify            *
+# * otherwise) that you are offering the Nmap Project the unlimited,        *
+# * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
+# * will always be available Open Source, but this is important because     *
+# * the inability to relicense code has caused devastating problems for     *
+# * other Free Software projects (such as KDE and NASM).  We also           *
 # * occasionally relicense the code to third parties as discussed above.    *
 # * If you wish to specify special license conditions of your               *
 # * contributions, just say so when you send them.                          *
@@ -145,19 +152,19 @@ from zenmapGUI.ScriptInterface import *
 
 def get_option_check_auxiliary_widget(option, ops, check):
     if option in ("-sI", "-b", "--script", "--script-args", "--exclude", "-p",
-        "-D", "-S", "--source-port", "-e", "--ttl", "-iR", "--max-retries",
-        "--host-timeout", "--max-rtt-timeout", "--min-rtt-timeout",
-        "--initial-rtt-timeout", "--max-hostgroup", "--min-hostgroup",
-        "--max-parallelism", "--min-parallelism", "--max-scan-delay",
-        "--scan-delay", "-PA", "-PS", "-PU", "-PO", "-PY"):
+            "-D", "-S", "--source-port", "-e", "--ttl", "-iR", "--max-retries",
+            "--host-timeout", "--max-rtt-timeout", "--min-rtt-timeout",
+            "--initial-rtt-timeout", "--max-hostgroup", "--min-hostgroup",
+            "--max-parallelism", "--min-parallelism", "--max-scan-delay",
+            "--scan-delay", "-PA", "-PS", "-PU", "-PO", "-PY"):
         return OptionEntry(option, ops, check)
     elif option in ("-d", "-v"):
         return OptionLevel(option, ops, check)
     elif option in ("--excludefile", "-iL"):
         return OptionFile(option, ops, check)
     elif option in ("-A", "-O", "-sV", "-n", "-6", "-Pn", "-PE", "-PP", "-PM",
-        "-PB", "-sC", "--script-trace", "-F", "-f", "--packet-trace", "-r",
-        "--traceroute"):
+            "-PB", "-sC", "--script-trace", "-F", "-f", "--packet-trace", "-r",
+            "--traceroute"):
         return None
     elif option in ("",):
         return OptionExtras(option, ops, check)
@@ -496,9 +503,8 @@ class OptionBuilder(object):
         return dic
 
     def __parse_groups(self):
-        return [g_name.getAttribute(u'name') for g_name in \
-                  self.xml.getElementsByTagName(u'groups')[0].\
-                  getElementsByTagName(u'group')]
+        return [g_name.getAttribute(u'name') for g_name in
+                self.xml.getElementsByTagName(u'groups')[0].getElementsByTagName(u'group')]  # noqa
 
     def __parse_tabs(self):
         dic = {}
